Package: cyjShiny
Title: Cytoscape.js Shiny Widget (cyjShiny)
Version: 1.0.42
Date: 2023-03-28
Authors@R: c(person("Omar", "Shah", role = c("ctb")), 
             person("Paul", "Shannon", email = "pshannon@systemsbiology.org", role = c("aut")), 
             person("Augustin", "Luna", email = "augustin_luna@hms.harvard.edu", role = c("aut", "cre"), comment = c(ORCID = "0000-0001-5709-371X")))       
Description: Wraps cytoscape.js as a shiny widget. cytoscape.js <https://js.cytoscape.org/> is a Javascript-based graph theory (network) library for visualization and analysis. This package supports the visualization of networks with custom visual styles and several available layouts. Demo Shiny applications are provided in the package code.  
Depends: R (>= 3.5.0), htmlwidgets, shiny, jsonlite, graph, base64enc
Suggests: RUnit, knitr, rmarkdown, markdown, BiocStyle
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: false
Collate: 'graphsToJSON.R' 'cyjShiny.R'
VignetteBuilder: knitr
RoxygenNote: 7.2.0
NeedsCompilation: no
Packaged: 2023-03-28 19:15:32 UTC; user
Author: Omar Shah [ctb],
  Paul Shannon [aut],
  Augustin Luna [aut, cre] (<https://orcid.org/0000-0001-5709-371X>)
Maintainer: Augustin Luna <augustin_luna@hms.harvard.edu>
Repository: CRAN
Date/Publication: 2023-03-28 21:00:02 UTC
Built: R 4.6.0; ; 2026-04-26 17:26:12 UTC; unix
