We use this directory for developer documentation and notes.
User documentation and notes, in addition to those generated by roxygen2, will appear in the wiki.

'''
Supplementary layouts added:

1. Cloned the repo
2. Went inside browsercore/src/cyjshiny.js
3. Added the npm dependency e.g. 
let spread = require('cytoscape-spread');
cytoscape.use(spread);
4. Also added the name of the new layout inside the doLayout function in cyjshiny.R file present at R/cyjshiny.R path
5. Executed the make file to build the bundle.js

 ''' 